#include "cTilemap.h"

#include "cImage.h"
#include "cAnimation.h"

cTilemap::cTilemap(char *filename)
{
	m_loaded = false;
	Load(filename);
}

cTilemap::~cTilemap()
{
	Unload();
}

void
cTilemap::Draw(int x, int y)
{
    int leftTile = x/64;
    int topTile = y/64;
    int leftOffset = x % 64;
    int topOffset = y % 64;

    for (int horiz_idx = 0; horiz_idx < 14; horiz_idx++) {
        for (int vert_idx = 0; vert_idx < 11; vert_idx++) {
            int mapOffset = (topTile+vert_idx)*m_xTiles+(leftTile+horiz_idx);

            if (map[mapOffset] == BROKEN_TILE) {
                map[mapOffset] = 0;
                m_breakAnim->Play(x+horiz_idx*64-leftOffset,y+vert_idx*64-topOffset);
            }

            tile[map[mapOffset]]->Draw(horiz_idx*64-leftOffset,vert_idx*64-topOffset);

        }
    }

    m_breakAnim->setOffset(x, y);
    m_breakAnim->Update();
    m_breakAnim->Draw();
}

void
cTilemap::BreakTile(int x, int y)
{
	if (isBreakable(x, y))
		map[(x/64)+(y/64)*m_xTiles] = BROKEN_TILE;
}

int
cTilemap::getTileIndex(int x, int y)
{
	if (x < 0 || x >= m_xTiles*64 || y < 0 || y >= m_yTiles*64)
		return -1;

	return (map[(x/64) + (y/64)*m_xTiles]);
}

bool
cTilemap::Load(char *filename)
{
	if (m_loaded)
		Unload();

	FILE *fp;
    char dummy[20], loadFile[50];

    if (!(fp = fopen(filename, "rt")))
		return false;

	int version;

	fscanf(fp, "%s", dummy);//Version:
	fscanf(fp, "%d", &version);

	fscanf(fp, "%s", dummy);//StartPoint:
	fscanf(fp, "%d", &m_startX);
	fscanf(fp, "%d", &m_startY);

	fscanf(fp, "%s", dummy);//StartPoint:
	fscanf(fp, "%d", &m_goalTileX);
	fscanf(fp, "%d", &m_goalTileY);

	fscanf(fp, "%s", dummy);// Animation:
	fscanf(fp, "%s", loadFile);
	m_breakAnim = new cAnimation(loadFile);

	fscanf(fp, "%s", dummy);// NumPics:
	fscanf(fp, "%d", &m_numTiles);

	for (int i = 0; i < m_numTiles; i++) {
		fscanf(fp, "%s", dummy);//Pic#:
		fscanf(fp, "%s", loadFile);
		tile[i] = new cImage(loadFile);
	}

	fscanf(fp, "%s", dummy);// NumBreakable:
	fscanf(fp, "%d", &m_numBreakable);

	fscanf(fp, "%s", dummy);// Breakable:
	for (int i = 0; i < m_numBreakable; i++)
		fscanf(fp, "%d", &m_breakable[i]);

	fscanf(fp, "%s", dummy);// NumImpassable:
	fscanf(fp, "%d", &m_numImpassable);

	fscanf(fp, "%s", dummy);// Impassable:
	for (int i = 0; i < m_numImpassable; i++)
		fscanf(fp, "%d", &m_impassable[i]);

	switch (version)
	{
	case 1:

		fscanf(fp, "%s", dummy);//Dimensions:
		fscanf(fp, "%d", &m_xTiles);
		fscanf(fp, "%d", &m_yTiles);

		fscanf(fp, "%s", dummy);//Tilemap:
		map = (int *)malloc((m_xTiles+1)*(m_yTiles+1)*sizeof(int));
		memset(map, 0, (m_xTiles+1)*(m_yTiles+1)*sizeof(int));

		for (int i = 0; i < m_xTiles*m_yTiles; i++) {
			fscanf (fp, "%d", &map[i]);

			if (map[i] < 0 || map[i] >= m_numTiles) {
				printf ("Map index %d has value %d. Out of range! (0, %d)\n",
						i, map[i], m_numTiles);
				map[i] = 0;
			}
		}
		break;
	case 2:
		// read image filename
		// load image
		// get info
		fscanf(fp, "%s", dummy);//Tilemap:
		fscanf (fp, "%s", loadFile);

		PALETTE pal;
		BITMAP *level = load_pcx(loadFile, pal);

		m_xTiles = level->w;
		m_yTiles = level->h;

		map = (int *)malloc((m_xTiles+1)*(m_yTiles+1)*sizeof(int));
		memset(map, 0, (m_xTiles+1)*(m_yTiles+1)*sizeof(int));

		for (int y = 0; y < m_yTiles; y++) {
			for (int x = 0; x < m_xTiles; x++) {
				int colour = getpixel(level, x, y);
				int index = bestfit_color(pal, getr(colour), getg(colour), getb(colour));

				if (index < m_numTiles)
					map[y*m_xTiles + x] = index;
				else
					map[y*m_xTiles + x] = 0;
			}
		}

		destroy_bitmap(level);
		break;
	}

	fscanf(fp, "%s", dummy);// NumLines:
	fscanf(fp, "%d", &m_numLines);

	m_maxLineLength = 0;
	for (int i = 0; i < m_numLines; i++) {
		fscanf(fp, "%s", m_line[i]);
		for (int j = 0; j < 76; j++) {
			if (m_line[i][j] == '_') {
				m_line[i][j] = ' ';
			}
		}

		for (int j = 0; j < 76; j++) {
			if (m_line[i][j] == '\0' && j > m_maxLineLength) {
				m_maxLineLength = j;
				break;
			}
		}
	}

	fclose(fp);

    m_loaded = true;

    return true;
}

void
cTilemap::Unload()
{
	if (!m_loaded)
		return;

	for (int i = 0; i < m_numTiles; i++) {
		delete tile[i];
		tile[i] = NULL;
	}

	delete m_breakAnim;
	free(map);

	m_loaded = false;
}

void
cTilemap::getStart(int *startX, int *startY) const
{
	*startX = m_startX;
	*startY = m_startY;
}

void
cTilemap::getGoalTile(int *goalTileX, int *goalTileY) const
{
	*goalTileX = m_goalTileX;
	*goalTileY = m_goalTileY;
}

bool
cTilemap::isGoal(int x, int y) const
{
	return (x/64 == m_goalTileX && y/64 == m_goalTileY);
}

bool
cTilemap::isBreakable(int x, int y) const
{
	for (int i = 0; i < m_numBreakable; i++)
		if (m_breakable[i] == map[x/64 + y/64*m_xTiles])
			return true;

	return false;
}

bool
cTilemap::isImpassable(int x, int y) const
{
	for (int i = 0; i < m_numImpassable; i++)
		if (m_impassable[i] == map[x/64 + y/64*m_xTiles])
			return true;

	return false;
}

int
cTilemap::getNumLines() const
{
	return m_numLines;
}

char
*cTilemap::getLine(int lineIndex)
{
	return m_line[lineIndex];
}

int
cTilemap::getMaxLineLength() const
{
	return m_maxLineLength;
}
